/**
 *	comsub.h  --  error code definition for comsub/comsubs
 *		Kazuro Furukawa <kazuro.furukawa@kek.jp>
 *
 *	K.Furukawa  Mar.2.1993.  extracted from comsub.c and comsubs.c
 *	K.F         Mar.2.1993.  for comsub v124->v130, comsubs v113-v120
 *	K.F         Jun.12.1993. add timeout error and type of functions
 *	K.F         Feb.21.1996. ansi C
 *
 * @(#)comsub.h v1.3.4  definition for comsub, K.Furukawa, Mar.1992 - Jan.2004.
 */

#ifndef __COMSUB_H
#define __COMSUB_H

#include "netdef.h"			/* this or system directory */

/* error codes */

#define S2_NOSERV	-1		/* service not found */
#define S2_NOPORT	-2		/* no port/service specified */
#define S2_NOHOST	-3		/* unknown host */
#define S2_NOPROT	-4		/* protocol other than tcp/udp */
#define S2_OPENERR	-5		/* socket open failure */
#define S2_UBNDERR	-6		/* socket bind failure */
#define S2_CONNERR	-7		/* socket connect failure */
#define S2_LISNERR	-8		/* socket listen failure */
#define S2_ACCPERR	-9		/* socket accept failure */
#define S2_FORKERR	-10		/* child fork failure */
#define S2_SENDERR	-21		/* send failure */
#define S2_RECVERR	-22		/* receive failure */
#define S2_SLCTERR	-23		/* select failure */
#define S2_TIMEOUT	-24		/* time out failure */

/* error/information message codes */

#define S2_ONONE	0		/* no message */
#define S2_OERR		1		/* error message */
#define S2_OINFO	2		/* information message */
#define S2_OALL		3		/* all message */

/* functions */

#ifdef __STDC__				/* for ansi C */

					/* in comsub.c */
int s2_open( int, char *, char *, char *, int );
int s2_bopen( int, char *, char *, char *, int );
int s2_scopen( char *, char *, int );
int s2_close( int, int );
int s2_send( int, char *, int, int );
int s2_receive( int, char *, int, int );
int s2_bsend( int, char *, int, int );
int s2_currtime1( time_t *, int * );
char *s2_currtime( void );
int s2_usleep( int );
int s2_sleep( int );
/* void s2_errmsg( int, char *, char * ); */
					/* in comsubs.c */
int s2_sopen( int, char *, char *, int );
int s2_accept( int, int, int );
int s2_sclose( int, int );
int s2_ureceive( int, char *, int, int );
int s2_usend( int, char *, int, int );
int s2_peerinfo( int, char **, int *, int );
char * s2_upeername( int, int );
int s2_upeerport( int, int );
					/* in comsubt.c */
int s2_topen( int, char *, char *, char *, int, int );
int s2_tsend( int, char *, int, int, int );
int s2_treceive( int, char *, int, int, int );
int s2_tusend( int, char *, int, int, int );
int s2_tureceive( int, char *, int, int, int );
					/* in comsubu.c */
int s2_uopen( int, char *, char *, char *, int );
int s2_scuopen( char *, char *, int );

#else  /* __STDC__ */			/* for Kernighan & Ritchie C */

					/* in comsub.c */
int s2_open( );
int s2_bopen( );
int s2_scopen( );
int s2_close( );
int s2_send( );
int s2_receive( );
int s2_bsend( );
int s2_currtime1( );
char *s2_currtime( );
int s2_usleep( );
int s2_sleep( );
/* void s2_errmsg( ); */
					/* in comsubs.c */
int s2_sopen( );
int s2_accept( );
int s2_sclose( );
int s2_ureceive( );
int s2_usend( );
int s2_peerinfo( );
char * s2_upeername( );
int s2_upeerport( );
					/* in comsubt.c */
int s2_topen( );
int s2_tsend( );
int s2_treceive( );
int s2_tusend( );
int s2_tureceive( );
					/* in comsubu.c */
int s2_uopen( );
int s2_scuopen( );

#endif  /* __STDC__ */

#endif /* __COMSUB_H */
