/**
 *	becho  --  remote echo program (broadcast version)
 *		Copyright (C) Kazuro Furukawa, 1988-1993
 *		Kazuro Furukawa, KEK, Japan <kazuro.furukawa@kek.jp>
 *
 *	K.Furukawa  Nov.9.1988.  (rwrite)
 *      K.F  Apr.13.1989.  converted from rwrite (recho)
 *	K.F  Jun.17.1992.  recho v3.4.4
 *	K.F  Oct.2.1992.   s2_recho using s2_xxx routines
 *	K.F  Nov.26.1992.  s2_recho v1.1.0
 *	K.F  Jun.12.1993.  s2_becho using s2_bopen and s2_treceive
 *	Modifications
 *	K.F  Jun.15.1993.  using s2_upeername
 *	K.F  Jul.12.1993.  use fgets instead of gets for os9
 *	K.F  Feb.21.1996.  ansi C
 *	K.F  Dec.16.2006.  sleep interval and timout in microsecond
 *
 *	This program sends messages typed on the terminal and receives 
 *	the echoed reply from s2_rechod program.  This will be the typical
 *	example for the TCP|UDP/IP server-client model.  By modifying
 *	{tcp|tcp}server() of Rechod, Recho and Rechod function as any 
 *	information server.
 *	With the loop option Recho loops to measure average response.
 *	See argument() for options.
 *
 *	This version uses s2_bopen() for broadcast and s2_treceive() for
 *	timeout handling, while s2_recho uses s2_open() and s2_receive().
 *	It broadcasts a message with udp onto the LAN and waits for 
 *	messages from the clients.  If s2_treceive() timeouts, it stops
 *	waiting.
 *	example:
 *	host1: s2_rechod -u
 *	host2: s2_rechod -u
 *	host3: s2_rechod -u
 *	host0: s2_becho 192.153.107.255
**/

char	what[] = 
"@(#)s2_becho.c v1.3.0 udp/ip brdcst. client, K.Furukawa, Nov.1988-Dec.2006";

#include "comsub.h"			/* network definitions */

#define  SERVICENAME	"s2_recho"	/* service name */
#define  PORTNUMB	11061		/* service port number */
#define  BUFFERSIZE	16384		/* communication buffer size */
#define  TIMEOUT	30		/* timeout for communication, 30ms */

char	buf[BUFFERSIZE];		/* i/o buffer */
int	sd;				/* file descriptor for socket com. */
char	*nm;				/* program name holder */
int	usetcp;				/* flag for protocol tcp/udp */
int	quietmode;			/* quiet mode */
int	portnum;			/* port number */
int	loopcount;			/* count for loop option */
int	loopsize;			/* buffer size for loop option */
long	sleeptime;			/* sleep time interval in micro-sec */
int	timeout;			/* timeout in milli-second */
void	errormsg( char * );

/**
 *  main
 *	Main prepares a connection socket for the recho service and starts
 *	client.
**/
int  main( int argc, char ** argv )
{
    char *argument( int, char ** );
    int client( void );
    char *host;					/* destination host name */

    nm = argv[0];				/* set program name */
    usetcp = 0;					/* use udp only */
    quietmode = 0;				/* verbose mode */
    portnum = 0;				/* port number */
    loopcount = 0;				/* no loop option by default */
    loopsize = 1;				/* size 1 by default */
    sleeptime = 0;				/* no sleep by default */
    timeout = TIMEOUT;				/* default timeout */

    host = argument(argc, argv);		/* command line argument */

    if ( !portnum )  {				/* port number not specified */
	portnum = -PORTNUMB;			/* default port */
    }

    sd = s2_bopen(portnum, SERVICENAME, (usetcp?"tcp":"udp"), host, S2_OALL);
    if ( sd<0 )  exit( 1 );			/* connect to server */

    return client();
}

/**
 *  client
 *	Client reads standard input, sends it to the server, receives
 *	its reply and displays it.
**/
int  client( void )
{
    int	bufcnt = 0, bufcnt2;			/* buffer length */
    long lines = 0;				/* communication counter */
    long bytes = 0;				/* communication counter */
    long liner = 0;				/* recieve counter */
    long byter = 0;				/* receive counter */
    int	i;					/* loop counter */

    if ( !loopcount )  {			/* no loop option */

	while( fgets( buf, BUFFERSIZE, stdin ) != NULL )  {
	    bufcnt = strlen( buf );		/* read standard input */
	    if( s2_bsend(sd, buf, bufcnt, S2_OALL) == bufcnt )  {
						/* broadcast to the servers */
		do  {
		    if( (bufcnt2 = s2_tureceive(sd, buf, BUFFERSIZE, 
					       timeout, S2_OERR)) > 0 )  {
						/* receive from a server */
			if (!quietmode)
			    (void) s2_upeername(sd, S2_OALL);
			write( 2, buf, bufcnt2 );
			if (bufcnt != bufcnt2)
			    errormsg( "send/receive size different" );
			liner++;
			byter += bufcnt2;
		    }
		}  while (bufcnt2 > 0);
	    }  else  {
		errormsg( "send size different" );
		exit( 1 );
	    }
	    lines++;
	    bytes += bufcnt;
	}
    
    }  else  {

	char  *c = buf;				/* buffer pointer */

	for ( i = 0 ; i < loopsize ; i++ )  *c++ = 'a';
	bufcnt = loopsize;

	while( lines < loopcount )  {		/* loop specified times */
	    if( s2_bsend(sd, buf, bufcnt, S2_OALL) == bufcnt )  {
						/* broadcast to the servers */
		do  {
		    if ((bufcnt2 = s2_tureceive(sd, buf, BUFFERSIZE, 
					      timeout, S2_OERR)) > 0 ) {
						/* receive from a server */
			if (bufcnt != bufcnt2)
			    errormsg( "send/receive size different" );
			liner++;
			byter += bufcnt2;
		    }
		}  while (bufcnt2 > 0);
	    }  else  {
		errormsg( "send size different" );
		exit( 1 );
	    }
	    lines++;
	    bytes += bufcnt;
	    if ( sleeptime )  s2_usleep( sleeptime );
	}

    }

    fprintf( stderr, 
	    "  %ld lines %ld bytes sent, %ld lines %ld bytes received\n", 
	    lines, bytes, liner, byter );
    s2_close( sd, S2_OALL );

    return 0;
}


/*
 *  errormsg
 *	print error message
 */
void  errormsg( char * msg )
{
	perror( nm );
	fprintf( stderr, "%s: %s\n", nm, msg );
}

/**
 *  argument
 *	Argument handles command line argument, such as hostname or options.
**/
char * argument( int argc, char ** argv )
{
	int c;
	extern int optind, opterr;
	extern char *optarg;
	opterr = 0;			/* disable informational message */

	while ((c = getopt (argc, argv, "uUp:P:l:L:s:S:i:I:o:O:qQhH")) != EOF)
	    switch ( c )  {
	    case 'u':
	    case 'U':				/* use udp */
		usetcp = 0;			/* reset tcp flag */
		break;
	    case 'q':
	    case 'Q':				/* quiet mode */
		quietmode = 1;			/* set quiet flag */
		break;
	    case 'p':
	    case 'P':				/* port number */
		portnum = atoi( optarg );
		break;
	    case 'l':
	    case 'L':				/* loop count */
		loopcount = atoi( optarg );
		break;
	    case 's':
	    case 'S':				/* loop buffer size */
		loopsize = atoi( optarg );
		break;
	    case 'i':				/* sleep interval time */
		sleeptime = atoi( optarg ) * 1000000;
		break;
	    case 'I':				/* sleep interval in micro */
		sleeptime = atol( optarg );
		break;
	    case 'o':
	    case 'O':				/* timeout value */
		timeout = atoi( optarg );
		break;
	    case '?':
	    case 'h':
	    case 'H':				/* help */
	    default:
		goto usage;			/* print out usage */
	    }
	if ( (argc-optind+1) > 1 )
	    return argv[optind];		/* host name */

usage:
	fprintf(stderr, "%s\n", what );
	fprintf(stderr, "Usage:  s2_becho  <option> <hostname>\n" );
	/* fprintf(stderr, " option:  -u   use UDP instead of TCP\n" ); */
	fprintf(stderr, " option:  -pX  use service port number X\n" );
	fprintf(stderr, "          -lY  loop Y times\n" );
	fprintf(stderr, "          -sZ  send Z bytes at a time (default=1)\n");
	fprintf(stderr, "          -iW  sleep W seconds interval\n");
	fprintf(stderr, "          -IW  sleep W micro-seconds interval\n");
	fprintf(stderr, "          -oU  timeout U milli-seconds\n");
	fprintf(stderr, "          -q   quiet mode\n" );
	fprintf(stderr, "          -h   display help message\n" );
	fprintf(stderr, " example: s2_becho 192.153.107.255\n" );
	fprintf(stderr, 
		"          s2_becho -o10 -p2000 -l100 -s10 130.87.160.0\n" );
	exit( 1 );
}
