/**
 *	oecho  --  remote echo program (open/close version)
 *		Copyright (C) Kazuro Furukawa, 1988-1994
 *		Kazuro Furukawa, KEK, Japan <kazuro.furukawa@kek.jp>
 *
 *	K.Furukawa  Nov.9.1988.  (rwrite)
 *      K.F  Apr.13.1989.  converted from rwrite (recho)
 *	K.F  Jun.17.1992.  recho v3.4.4
 *	K.F  Oct.2.1992.   s2_recho using s2_xxx routines
 *	K.F  Jul.12.1993.  s2_recho v1.2.0
 *	Modifications
 *	K.F  Feb.21.1996.  ansi C
 *
 *	This program sends messages typed on the terminal and receives 
 *	the echoed reply from s2_rechod program.  This will be the typical
 *	example for the TCP|UDP/IP server-client model.  By modifying
 *	{tcp|tcp}server() of Rechod, Recho and Rechod will function as 
 *	any information server.
 *
 *	This version of Recho, Oecho, opens and closes the socket every 
 *	time it sends data.
 *
 *	With the loop option Oecho loops to measure average response.
 *	It measures socket open/(connect)/close performance.
 *
 *	See argument() for options.
**/

char	what[] = 
"@(#)s2_oecho.c v1.1.3 tcp|udp/ip echo client, K.Furukawa, Nov.1988-Aug.2003";

#include "comsub.h"			/* network definitions */

#define  SERVICENAME	"s2_recho"	/* service name */
#define  PORTNUMB	11061		/* service port number */
#define  BUFFERSIZE	16384		/* communication buffer size */

char	buf[BUFFERSIZE];		/* i/o buffer */
int	sd;				/* file descriptor for socket com. */
char	*nm;				/* program name holder */
int	usetcp;				/* flag for protocol tcp/udp */
int	portnum;			/* port number */
int	loopcount;			/* count for loop option */
int	loopsize;			/* buffer size for loop option */
int	sleeptime;			/* sleep time for interval option */
void	errormsg( char * );
char	*host;				/* destination host name */

int  oecho_send( int, char *, int, int );
int  oecho_receive( int, char *, int, int );

/**
 *  main
 *	Main prepares a connection socket for the recho service and starts
 *	client.
**/
int  main( int argc, char ** argv )
{
    char *argument( int, char ** );
    int client( void );

    nm = argv[0];				/* set program name */
    usetcp = 1;					/* use tcp by default */
    portnum = 0;				/* port number */
    loopcount = 0;				/* no loop option by default */
    loopsize = 1;				/* size 1 by default */
    sleeptime = 0;				/* no sleep by default */

    host = argument(argc, argv);		/* command line argument */

    if ( !portnum )  {				/* port number not specified */
	portnum = -PORTNUMB;			/* default port */
    }

    /*
    sd = s2_open( portnum, SERVICENAME, (usetcp?"tcp":"udp"), host, S2_OALL );
    if ( sd<0 )  exit( 1 );
    */
						/* connect to server */

    return client();
    /* s2_close( sd, S2_OALL ); */
}

/**
 *  client
 *	Client reads standard input, sends it to the server, receives
 *	its reply and displays it.
**/
int  client( void )
{
    int	bufcnt = 0;				/* buffer length */
    long lines = 0;				/* communication counter */
    long bytes = 0;				/* communication counter */
    int	i;					/* loop counter */

    if ( !loopcount )  {			/* no loop option */

	while( fgets( buf, BUFFERSIZE, stdin ) != NULL )  {
	    int bufcnt2 = 0;			/* buffer length */
	    bufcnt = strlen( buf );		/* read standard input */
	    if( oecho_send(sd, buf, bufcnt, S2_OALL) == bufcnt )  {
						/* send to the server */
		if( (bufcnt2 = oecho_receive(sd, buf, BUFFERSIZE, 
					     S2_OALL)) > 0 )
		    				/* receive from the server */
		    write( 2, buf, bufcnt2 );
		if (bufcnt != bufcnt2)
		    errormsg( "send/receive size different" );
	    }  else  {
		errormsg( "send size different" );
		exit( 1 );
	    }
	    lines++;
	    bytes += bufcnt2;
	}
    
    }  else  {

	char  *c = buf;				/* buffer pointer */

	fprintf( stderr, " Program starts at %s\n", s2_currtime() );

	for ( i = 0 ; i < loopsize ; i++ )  *c++ = 'a';
	bufcnt = loopsize;

	while( lines < loopcount )  {		/* loop specified times */
	    int bufcnt2 = 0;			/* buffer length */

	    if( oecho_send(sd, buf, bufcnt, S2_ONONE) == bufcnt )  {
						/* send to the server */
		if ((bufcnt2 = oecho_receive(sd, buf, BUFFERSIZE, 
					  S2_ONONE)) != bufcnt) {
		    				/* receive from the server */
		    errormsg( "send/receive size different" );
		}
	    }  else  {
		errormsg( "send size different" );
		exit( 1 );
	    }
	    lines++;
	    bytes += bufcnt2;
	    if ( sleeptime )  sleep( sleeptime );
	}

	fprintf( stderr, " Program exits at %s\n", s2_currtime() );
    }

    fprintf( stderr, "  %ld lines %ld bytes\n", lines, bytes );
    return 0;
}


/*
 *  errormsg
 *	print error message
 */
void  errormsg( char * msg )
{
	perror( nm );
	fprintf( stderr, "%s: %s\n", nm, msg );
}

/**
 *  argument
 *	Argument handles command line argument, such as hostname or options.
**/
char *  argument( int argc, char ** argv )
{
	int c;
	extern int optind, opterr;
	extern char *optarg;
	opterr = 0;			/* disable informational message */

	while ((c = getopt (argc, argv, "uUp:P:l:L:s:S:i:I:hH")) != EOF)
	    switch ( c )  {
	    case 'u':
	    case 'U':				/* use udp */
		usetcp = 0;			/* reset tcp flag */
		break;
	    case 'p':
	    case 'P':				/* port number */
		portnum = atoi( optarg );
		break;
	    case 'l':
	    case 'L':				/* loop count */
		loopcount = atoi( optarg );
		break;
	    case 's':
	    case 'S':				/* loop buffer size */
		loopsize = atoi( optarg );
		break;
	    case 'i':
	    case 'I':				/* sleep interval time */
		sleeptime = atoi( optarg );
		break;
	    case '?':
	    case 'h':
	    case 'H':				/* help */
	    default:
		goto usage;			/* print out usage */
	    }
	if ( (argc-optind+1) > 1 )
	    return argv[optind];		/* host name */

usage:
	fprintf(stderr, "%s\n", what );
	fprintf(stderr, "Usage:  s2_recho  <option> <hostname>\n" );
	fprintf(stderr, " option:  -u   use UDP instead of TCP\n" );
	fprintf(stderr, "          -pX  use service port number X\n" );
	fprintf(stderr, "          -lY  loop Y times\n" );
	fprintf(stderr, "          -sZ  send Z bytes at a time (default=1)\n");
	fprintf(stderr, "          -iW  sleep W seconds interval\n");
	fprintf(stderr, "          -h   display help message\n" );
	fprintf(stderr, " example: s2_recho 130.87.160.10\n" );
	fprintf(stderr, "          s2_recho -u -p2000 -l100 -s10 apricot\n" );
	exit( 1 );
}

/**
 *  oecho_send
 *	open socket and send data.
**/
int  oecho_send( int ossd, char * osbuf, int osbufcnt, int osopt)
  /* ossd is dummy */
{
    sd = s2_open( portnum, SERVICENAME, (usetcp?"tcp":"udp"), 
		   host, osopt );
    if ( sd < 0 )  return sd;

    return s2_send(sd, osbuf, osbufcnt, osopt);
}

/**
 *  oecho_receive
 *	receive data and close socket.
**/
int  oecho_receive( int ossd, char * osbuf, int osbufcnt, int osopt )
  /* ossd is dummy */
{
    int bufcnt, stat;

    bufcnt = s2_receive(sd, osbuf, osbufcnt, osopt);
    if (bufcnt < 0)  return bufcnt;

    stat = s2_close( sd, osopt );
    if (stat < 0)  return stat;

    return bufcnt;
}

