/**
 *	trecho  --  remote echo program (with timeout)
 *		Copyright (C) Kazuro Furukawa, 1988-2002
 *		Kazuro Furukawa, KEK, Japan <kazuro.furukawa@kek.jp>
 *
 *	K.Furukawa  Nov.9.1988.  (rwrite)
 *      K.F  Apr.13.1989.  converted from rwrite (recho)
 *	K.F  Jun.17.1992.  recho v3.4.4
 *	K.F  Oct.2.1992.   s2_recho using s2_xxx routines
 *	K.F  Nov.26.1992.  s2_recho v1.1.0
 *	K.F  Jun.12.1993.  s2_techo using s2_t{send|receive}
 *	Modifications
 *	K.F  Jul.12.1993.  use fgets instead of gets for os9
 *	K.F  Aug.29.1995.  use s2_topen instead of s2_open
 *	K.F  Feb.21.1996.  ansi C
 *	K.F  May.19.1999.  for vxworks
 *	K.F  Mar.3.2000.   add no-check (-c) and no-newline (-n) options
 *	K.F  Apr.10.2002.  add and-carriage-return (-r) option
 *
 *	This program sends messages typed on the terminal and receives 
 *	the echoed reply from s2_rechod program.  This will be the typical
 *	example for the TCP|UDP/IP server-client model.  By modifying
 *	{tcp|tcp}server() of Techod, Techo and Techod will function as any 
 *	information server-client.
 *	With the loop option Recho loops to measure average response.
 *	See argument() for options.
 *	This version uses s2_topen(), s2_tsend() and s2_treceive() with 
 *	timeout, while s2_recho uses s2_open(), s2_send() and s2_receive().
**/

char	what_s2techo[] = 
"@(#)s2_techo.c v1.4.3 tcp|udp/ip echo client, K.Furukawa, Nov.1988-Jan.2004";

#include "comsub.h"			/* network definitions */

#define  SERVICENAME	"s2_recho"	/* service name */
#define  PORTNUMB	11061		/* service port number */
#define  BUFFERSIZE	16384		/* communication buffer size */
#define  TIMEOUT	100		/* timeout for communication, 100ms */

static char	buf[BUFFERSIZE];	/* i/o buffer */
static char	*nm;			/* program name holder */
static int	usetcp;			/* flag for protocol tcp/udp */
static int	addcr;			/* flag for packets with cr */
static int	nonl;			/* flag for packets without new-line */
static int	nocheck;		/* flag to checking of reply */
static int	portnum;		/* port number */
static int	loopcount;		/* count for loop option */
static int	loopsize;		/* buffer size for loop option */
static int	sleeptime;		/* sleep time for interval option */
static int	timeout;		/* timeout in milli-second */
static void	errormsg( char * );
static char *	argument( int, char ** );
static int	client( int );

/**
 *  main
 *	Main prepares a connection socket for the recho service and starts
 *	client.
**/
#ifdef vxWorks
static int  s2_techo_main( int argc, char ** argv )
#else /* vxWorks */
int  main( int argc, char ** argv )
#endif /* vxWorks */

{
    char *host;					/* destination host name */
    int	sd;					/* file descriptor for socket */

    nm = argv[0];				/* set program name */
    usetcp = 1;					/* use tcp by default */
    addcr = 0;					/* without cr by default */
    nonl = 0;					/* with new-line by default */
    nocheck = 0;				/* check reply by default */
    portnum = 0;				/* port number */
    loopcount = 0;				/* no loop option by default */
    loopsize = 1;				/* size 1 by default */
    sleeptime = 0;				/* no sleep by default */
    timeout = TIMEOUT;				/* default timeout */

    host = argument(argc, argv);		/* command line argument */

    if ( !host )  return 1;
    
    if ( !portnum )  {				/* port number not specified */
	portnum = -PORTNUMB;			/* default port */
    }

    sd = s2_topen(portnum, SERVICENAME, (usetcp?"tcp":"udp"), 
		  host, timeout, S2_OALL);
    if ( sd<0 )  return 1;			/* connect to server */

    return client( sd );
}

/**
 *  client
 *	Client reads standard input, sends it to the server, receives
 *	its reply and displays it.
**/
static int client( int sd )
{
    int	bufcnt = 0;				/* buffer length */
    long lines = 0;				/* communication counter */
    long bytes = 0;				/* communication counter */
    int	i;					/* loop counter */

    if ( !loopcount )  {			/* no loop option */

	while( fgets( buf, BUFFERSIZE, stdin ) != NULL )  {
	    int bufcnt2 = 0;			/* buffer length */
	    bufcnt = strlen( buf );		/* read standard input */
	    if( nonl )  bufcnt--;		/* remove trailing new-line */
	    if( addcr )  buf[bufcnt++]='\r';	/* add trailing cr */
	    if( s2_tsend(sd, buf, bufcnt, timeout, S2_OALL) == bufcnt )  {
						/* receive from the server */
		if( (bufcnt2 = s2_treceive(sd, buf, BUFFERSIZE, 
					   timeout, S2_OALL)) > 0 )  {
		    				/* receive from the server */
		    write( 2, buf, bufcnt2 );
		    if( nonl )  write( 2, "\n", 1 );	/* local new-line */
		}
		if (!nocheck && bufcnt != bufcnt2)
		    errormsg( "send/receive size different" );
	    }  else  {
		errormsg( "send size different" );
		return 1;
	    }
	    lines++;
	    if ( bufcnt2>0 ) bytes += bufcnt2;
	}
    
    }  else  {

	char  *c = buf;				/* buffer pointer */

	for ( i = 0 ; i < loopsize ; i++ )  *c++ = 'a';
	bufcnt = loopsize;

	while( lines < loopcount )  {		/* loop specified times */
	    int bufcnt2 = 0;			/* buffer length */

	    if( s2_tsend(sd, buf, bufcnt, timeout, S2_OALL) == bufcnt )  {
						/* send to the server */
		if ((bufcnt2 = s2_treceive(sd, buf, BUFFERSIZE, 
					  timeout, S2_OALL)) != bufcnt) {
		    				/* receive from the server */
		    if ( !nocheck )  errormsg( "send/receive size different" );
		}
	    }  else  {
		errormsg( "send size different" );
		return 1;
	    }
	    lines++;
	    if ( bufcnt2>0 )  bytes += bufcnt2;
	    if ( sleeptime )  s2_sleep( sleeptime );
	}

    }

    fprintf( stderr, "  %ld lines %ld bytes\n", lines, bytes );
    s2_close( sd, S2_OALL );

    return 0;
}


/*
 *  errormsg
 *	print error message
 */
static void  errormsg( char * msg )
{
	perror( nm );
	fprintf( stderr, "%s: %s\n", nm, msg );
}

#ifdef vxWorks
/*
 * command line interface for vxworks
 */
int s2_techo(char *s1,char *s2,char *s3,char *s4,char *s5,char *s6,char *s7)
{
    char *al[8];			/* argument list */
    int ac=1;				/* argument count */
    extern int optind;
    optind=1;				/* hack for vxworks keeping static */
    clearerr(stdin);
    if(s1){ac++;al[1]=s1; if(s2){ac++;al[2]=s2; if(s3){ac++;al[3]=s3; 
	if(s4){ac++;al[4]=s4; if(s5){ac++;al[5]=s5; if(s6){ac++;al[6]=s6;
	    if(s7){ac++;al[7]=s7;al[8]=0;
	}else{al[7]=0;} }else{al[6]=0;} }else{al[5]=0;} }else{al[4]=0;} 
    }else{al[3]=0;} }else{al[2]=0;} }else{al[1]=0;}
    al[0]="s2_techo";
    return s2_techo_main(ac,al);
}
#endif /* vxWorks */

/**
 *  argument
 *	Argument handles command line argument, such as hostname or options.
**/
static char *  argument( int argc, char ** argv )
{
	int c;
	extern int optind, opterr;
	extern char *optarg;
	opterr = 0;			/* disable informational message */

	while ((c = getopt(argc, argv, "uUnNrRcCp:P:l:L:s:S:i:I:o:O:hH")) != EOF)
	    switch ( c )  {
	    case 'u':
	    case 'U':				/* use udp */
		usetcp = 0;			/* reset tcp flag */
		break;
	    case 'r':
	    case 'R':				/* add carriage-return */
		addcr = 1;			/* set add cr flag */
		break;
	    case 'n':
	    case 'N':				/* no new-line */
		nonl = 1;			/* set no new-line flag */
		break;
	    case 'c':
	    case 'C':				/* no check */
		nocheck = 1;			/* set no check flag */
		break;
	    case 'p':
	    case 'P':				/* port number */
		portnum = atoi( optarg );
		break;
	    case 'l':
	    case 'L':				/* loop count */
		loopcount = atoi( optarg );
		break;
	    case 's':
	    case 'S':				/* loop buffer size */
		loopsize = atoi( optarg );
		break;
	    case 'i':
	    case 'I':				/* sleep interval time */
		sleeptime = atoi( optarg );
		break;
	    case 'o':
	    case 'O':				/* timeout value */
		timeout = atoi( optarg );
		break;
	    case '?':
	    case 'h':
	    case 'H':				/* help */
	    default:
		goto usage;			/* print out usage */
	    }
	if ( (argc-optind+1) > 1 )
	    return argv[optind];		/* host name */

usage:
	fprintf(stderr, "%s\n", what_s2techo );
	fprintf(stderr, "Usage:  s2_techo  <option> <hostname>\n" );
	fprintf(stderr, " option:  -u   use UDP instead of TCP\n" );
	fprintf(stderr, "          -pX  use service port number X\n" );
	fprintf(stderr, "          -c   no check for received packet\n" );
	fprintf(stderr, "          -n   no new-line at end-of-packet\n" );
	fprintf(stderr, "          -r   add carriage-retun at end\n" );
	fprintf(stderr, "          -lY  loop Y times\n" );
	fprintf(stderr, "          -sZ  send Z bytes at a time (default=1)\n");
	fprintf(stderr, "          -iW  sleep W seconds interval\n");
	fprintf(stderr, "          -oU  timeout U milli-seconds\n");
	fprintf(stderr, "          -h   display help message\n" );
	fprintf(stderr, " example: s2_techo 130.87.160.10\n" );
	fprintf(stderr, "          s2_techo -uo10 -p2000 -l100 -s10 maple\n" );
	return NULL;
}
