/**
 *	recho  --  remote echo program
 *		Copyright (C) Kazuro Furukawa, 1988-1993
 *		Kazuro Furukawa, KEK, Japan <kazuro.furukawa@kek.jp>
 *
 *	K.Furukawa  Nov.9.1988.  (rwrite)
 *      K.F  Apr.13.1989.  converted from rwrite to recho
 *      K.F  Mar.12.1991.  converted from recho to xfer
 *	K.F  Jun.17.1992.  recho v3.4.4
 *	K.F  Oct.2.1992.   s2_recho using s2_xxx routines
 *	K.F  Nov.26.1992.  s2_recho v1.1.0
 *	K.F  Jun.10.1993.  s2_xfer using s2_xxx routines
 *	Modifications
 *	K.F  Feb.21.1996.  ansi C
 *
 *	This program sends messages typed on the terminal to s2_xferd.
 *	S2_xfer and s2_xferd process one-way transmittion of messages 
 *	while s2_recho and s2_rechod communicate in bi-directional way.
 *	With tcp, s2_xfer can send all data from stdin to stdout of 
 *	s2_xferd.  Thus s2_xfer and s2_xferd act as a pipe through the 
 *	network.  With udp, s2_xfer may lose messages if the transfer 
 *	rate is high.  However with udp it can send messages to many 
 *	hosts with broadcast. 
 *	This will be the typical example for the TCP|UDP/IP server-client 
 *	model.
 *	With the loop option s2_xfer loops to measure average transfer rate.
 *	See argument() for options.
**/

char	what[] = 
"@(#)s2_xfer.c v1.1.1 tcp|udp/ip xfer client, K.Furukawa, Nov.1988-Aug.2003";

#include "comsub.h"			/* network definitions */

#define  SERVICENAME	"s2_xfer"	/* service name */
#define  PORTNUMB	11062		/* service port number */
#define  BUFFERSIZE	16384		/* communication buffer size */

char	buf[BUFFERSIZE];		/* i/o buffer */
int	sd;				/* file descriptor for socket com. */
char	*nm;				/* program name holder */
int	usetcp;				/* flag for protocol tcp/udp */
int	broadcast;			/* flag for broadcast */
int	portnum;			/* port number */
int	loopcount;			/* count for loop option */
int	loopsize;			/* buffer size for loop option */
int	sleeptime;			/* sleep time for interval option */
void	errormsg( char * );

/**
 *  main
 *	Main prepares a connection socket for the xfer service and starts
 *	client.
**/
int  main( int argc, char ** argv )
{
    char *argument( int, char ** );
    int client( void );
    char *host;					/* destination host name */

    nm = argv[0];				/* set program name */
    usetcp = 1;					/* use tcp by default */
    broadcast = 0;				/* no broadcast by default */
    portnum = 0;				/* port number */
    loopcount = 0;				/* no loop option by default */
    loopsize = 1;				/* size 1 by default */
    sleeptime = 0;				/* no sleep by default */

    host = argument(argc, argv);		/* command line argument */

    if ( !portnum )  {				/* port number not specified */
	portnum = -PORTNUMB;			/* default port */
    }

    if ( broadcast )
	sd = s2_bopen(portnum, SERVICENAME, (usetcp?"tcp":"udp"), 
		      host, S2_OALL);
    else
	sd = s2_open(portnum, SERVICENAME, (usetcp?"tcp":"udp"), 
		     host, S2_OALL);
    if ( sd<0 )  exit( 1 );			/* connect to server */

    return client();
}

/**
 *  client
 *	Client reads standard input, sends it to the server.
**/
int  client( void )
{
    int	bufcnt = 0;				/* buffer length */
    long lines = 0;				/* communication counter */
    long bytes = 0;				/* communication counter */
    int (*send_routine)();			/* send routine */

    if ( broadcast )				/* for broadcast type */
	send_routine = s2_bsend;		/* use special routine */
    else
	send_routine = s2_send;

    if ( !loopcount )  {			/* no loop option */

	/* while( gets( buf ) != NULL )  { */
	/* while( fgets(buf,BUFFERSIZE,stdin) != NULL )  { */
	while( (bufcnt = read(0, buf, BUFFERSIZE)) != 0 )  {
	    /* read standard input */
		if( send_routine(sd, buf, bufcnt, S2_OALL) == bufcnt )  {
		;
	    }  else  {
		errormsg( "send size different" );
		exit( 1 );
	    }
	    lines++;
	    bytes += bufcnt;
	}

    }  else  {

	int	i;				/* loop counter */
	char  *c = buf;				/* buffer pointer */

	for ( i = 0 ; i < loopsize ; i++ )  *c++ = 'a';
	bufcnt = loopsize;

	while( lines < loopcount )  {		/* loop specified times */
	    if( send_routine(sd, buf, bufcnt, S2_OALL) == bufcnt )  {
		;
	    }  else  {
		errormsg( "send size different" );
		exit( 1 );
	    }
	    lines++;
	    bytes += bufcnt;
	    if ( sleeptime )  sleep( sleeptime );
	}

    }

    fprintf( stderr, "  %ld lines %ld bytes\n", lines, bytes );
    s2_close( sd, S2_OALL );
    return 0;
}


/*
 *  errormsg
 *	print error message
 */
void  errormsg( char * msg )
{
	perror( nm );
	fprintf( stderr, "%s: %s\n", nm, msg );
}

/**
 *  argument
 *	Argument handles command line argument, such as hostname or options.
**/
char *  argument( int argc, char ** argv )
{
	int c;
	extern int optind, opterr;
	extern char *optarg;
	opterr = 0;			/* disable informational message */

	while ((c = getopt (argc, argv, "uUbBp:P:l:L:s:S:i:I:hH")) != EOF)
	    switch ( c )  {
	    case 'b':
	    case 'B':				/* use broadcast */
		broadcast = 1;			/* set broadcast & udp flag */
	    case 'u':
	    case 'U':				/* use udp */
		usetcp = 0;			/* reset tcp flag */
		break;
	    case 'p':
	    case 'P':				/* port number */
		portnum = atoi( optarg );
		break;
	    case 'l':
	    case 'L':				/* loop count */
		loopcount = atoi( optarg );
		break;
	    case 's':
	    case 'S':				/* loop buffer size */
		loopsize = atoi( optarg );
		break;
	    case 'i':
	    case 'I':				/* sleep interval time */
		sleeptime = atoi( optarg );
		break;
	    case '?':
	    case 'h':
	    case 'H':				/* help */
	    default:
		goto usage;			/* print out usage */
	    }
	if ( (argc-optind+1) > 1 )
	    return argv[optind];		/* host name */

usage:
	fprintf(stderr, "%s\n", what );
	fprintf(stderr, "Usage:  s2_xfer  <option> <hostname>\n" );
	fprintf(stderr, " option:  -u   use UDP instead of TCP\n" );
	fprintf(stderr, "          -b   use broadcast with UDP\n" );
	fprintf(stderr, "          -pX  use service port number X\n" );
	fprintf(stderr, "          -lY  loop Y times\n" );
	fprintf(stderr, "          -sZ  send Z bytes at a time (default=1)\n");
	fprintf(stderr, "          -iW  sleep W seconds interval\n");
	fprintf(stderr, "          -h   display help message\n" );
	fprintf(stderr, " example: s2_xfer 130.87.160.10\n" );
	fprintf(stderr, "          s2_xfer -u -p2000 -l100 -s10 apricot\n" );
	fprintf(stderr, "          s2_xfer -b 192.153.107.255\n" );
	exit( 1 );
}
